-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
function BlockWars:StartActivity()
	if self.IsInitialized == nil then
		self.IsInitialized = false
	end

	if self.IsInitialized then 
		return
	end

	-- If you are a developer you can turn on/off almost all of
	-- this mission's debug yada-yada with this flag
	self.Debug = false;
	self.DrawWaypoints = false --true;
	
   	print("START! -- BlockWarsGameplay:StartActivity()!");

	self.ModuleName = "blockwars.rte";
	
	if PresetMan:GetModuleID("DREDD.rte") ~= -1 then
		self.UseJudges = true
		self.JudgeLZ = SceneMan.Scene:GetArea("JudgeLZ");
	else
		self.UseJudges = false
	end
	
	if self.PlayerCount == 1 then
		for i = 1, 10 do
			if SceneMan.Scene:HasArea("Observation"..i) then
				local actor = CreateActor("Observer")
				if actor ~= nil then
					actor.Team = 0
					actor.Pos = SceneMan.Scene:GetArea("Observation"..i):GetCenterPoint()
					MovableMan:AddActor(actor)
				end
			else
				break;
			end
		end
	end
	
    self.FightPhase = { NOFIGHT = 0};
    self.CurrentFightPhase = self.FightPhase.NOFIGHT;

	self.BrainhuntPercentage = 0.20; -- Amount of defenders that will be sent to hunt for enemy brain.

	self.braindead = {};
	self.AssignedBrains = {};

	self.MOIDLimit = 220;
    self.GenericTimer = Timer();
	
   	-- Set up teams
   	-- Team 2 is always CPU
	self.PlayerTeam_1 = Activity.TEAM_1;
   	self.PlayerTeam_2 = Activity.TEAM_2;
	self.RogueTeam = -1;

	-- Send troops to specific not more than 1 unit\14	 secs
	self.DispatchInterval = 20000;
	
	-- Every once in a while all actors start searching for brainz
	self.RageInterval = 90000;
    self.RageTimer = Timer();


	-- Init team megastructures
	self.T = {}

	for team = 0, 1 do
		self.T[team] = {};
		self.T[team].SpawnTimer = Timer();
		self.T[team].SpawnTimer:Reset();
		self.T[team].SpawnInterval = 6500;
		self.T[team].SpawnCoeff = 1.0;
		
		
		self.T[team].Chamber = SceneMan.Scene:GetArea("Chamber-"..team + 1);
		self.T[team].BrainRoom = SceneMan.Scene:GetArea("BrainRoom-"..team + 1);

		self.T[team].Hall = SceneMan.Scene:GetArea("Hall-"..team + 1);
		self.T[team].HallDef = SceneMan.Scene:GetArea("HallDef-"..team + 1);

		self.T[team].ChamberEmpty = true;
		self.T[team].HallEmpty = true;
		self.T[team].ChamberEmptyLastTime = true;
		self.T[team].Dispatch = SceneMan.Scene:GetArea("Dispatch-"..team + 1);
		self.T[team].Brain = SceneMan.Scene:GetArea("Brain-"..team + 1);
		self.T[team].Spawn = SceneMan.Scene:GetArea("Spawn-"..team + 1);
		
		self.T[team].Def = {};
		self.T[team].Waypoint = {};
		self.T[team].DefEmpty = {};
		self.T[team].LastDispatch = {};
		
		for defence = 1, 6 do
			self.T[team].Def[defence] = SceneMan.Scene:GetArea("Defence-"..(team + 1).."-"..defence);
			self.T[team].DefEmpty[defence] = true;
			self.T[team].LastDispatch[defence] = -(self.DispatchInterval * 2);
		end

		for defence = 1, 5 do
			self.T[team].Waypoint[defence] = SceneMan.Scene:GetArea("Waypoint-"..(team + 1).."-"..defence);
		end
		
		self.TroopsCount = 0;
	end
	
	if SceneMan.Scene:HasArea("NeedDiggers") then
		self.DiggerProbability = 0.3
	else
		self.DiggerProbability = -1
	end
	
	self.T[self.PlayerTeam_1].Faction = self:GetTeamTech(self.PlayerTeam_1);
	self.T[self.PlayerTeam_2].Faction = self:GetTeamTech(self.PlayerTeam_2);

    for actor in MovableMan.AddedActors do
		if actor.Team ~= -1 then
			-- Replace soldier units
			if actor.PresetName == "Dummy" then
				self:SpawnRandomInfantry(actor.Team , actor.Pos , self.T[actor.Team].Faction , Actor.AIMODE_SENTRY);
				actor.Pos = Vector(SceneMan.Scene.Width, SceneMan.Scene.Height * 2)
			end

			-- Replace armor units
			if actor.PresetName == "Dreadnought" then
				self:SpawnArmor(actor.Team , actor.Pos , self.T[actor.Team].Faction , Actor.AIMODE_SENTRY);
				actor.Pos = Vector(SceneMan.Scene.Width, SceneMan.Scene.Height * 2)
			end
		end
    end
	
	-- Decision making or usefull stats
	self.UnitCount = 0;

    
   	for player = 0, self.PlayerCount - 1 do
		-- Check if we already have a brain assigned
		-- Set up players
		self.braindead[player] = false;
		local brain = CreateActor("Brain Case" , "Base.rte");
		brain.Team = self:GetTeamOfPlayer(player);
		brain.AIMode = Actor.AIMODE_SENTRY;
		brain.Pos = self.T[self:GetTeamOfPlayer(player)].Brain:GetCenterPoint() + Vector(0,-26);
		MovableMan:AddActor(brain);
		self.AssignedBrains[player] = brain;

		-- Set the found brain to be the selected actor at start
		self:SetPlayerBrain(brain, player);
		
		-- Set the observation target to the brain, so that if/when it dies, the view flies to it in observation mode
		self:SetObservationTarget(self:GetPlayerBrain(player).Pos, player)
		self:SwitchToActor(brain, player, self:GetTeamOfPlayer(player));
	end --for
	
	-- Create brain for CPU team
	if self.CPUTeam > -1 then
		self.CPUBrain = CreateActor("Brain Case" , "Base.rte");
		self.CPUBrain.Team = self.CPUTeam;
		self.CPUBrain.Pos = self.T[self.CPUTeam].Brain:GetCenterPoint() + Vector(0,-26);
		self.CPUBrain.AIMode = Actor.AIMODE_SENTRY;
		self.CPUBrainDead = false;
		
		MovableMan:AddActor(self.CPUBrain);
	end
	
    -- Set the team's funds.
    self:SetTeamFunds(0 , self.PlayerTeam_1);
    self:SetTeamFunds(0 , self.PlayerTeam_2);

    -- Reset all timers
	self.GenericTimer:Reset();
	self.RageTimer:Reset();
	
	self.JudgeInterval = 90000
	self.JudgeTimer = Timer()
	self.JudgeTimer:Reset();
	
	self.IsInitialized = true
end

-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
function BlockWars:PauseActivity(pause)
    print("PAUSE! -- DummyAssault:PauseActivity()!");
end

-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
function BlockWars:EndActivity()
    print("END! -- DummyAssault:EndActivity()!");
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function BlockWars:GetModuleName(s)
	local pos;
	
	pos = string.find(s ,"/");
	
	if pos ~= nil then
		local  preset , module;

		module = string.sub(s , 1 , pos - 1);
		preset = string.sub(s , pos + 1 , string.len(s));

		return module
	else
		return nil
	end
end

-----------------------------------------------------------------------------------------
-- Spawns armor of selected faction. Returns actor, price.
-----------------------------------------------------------------------------------------
function BlockWars:SpawnArmor(team , pos , faction , aimode)
	local actor = nil;
	local r;
	if MovableMan:GetMOIDCount() < self.MOIDLimit then
		actor = RandomACrab("Actors - Mecha", faction)
		if actor then
			actor.AIMode = aimode;
			actor.Team = team;
			
			if pos ~= nil then
				actor.Pos = pos;
				MovableMan:AddActor(actor);
			end
		else
			actor = RandomAHuman("Actors - Mecha", faction)
			if actor then
				actor.AIMode = aimode;
				actor.Team = team;
				
				if pos ~= nil then
					actor.Pos = pos;
					MovableMan:AddActor(actor);
				end
			else
				actor = self:SpawnRandomInfantry(team, pos, faction,aimode);
			end
		end
		return actor;
	end
	
	return nil;
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function BlockWars:SpawnRandomInfantry(team , pos , faction , aimode)
	local actor = nil;
	local r;
	local item;
	
	if MovableMan:GetMOIDCount() < self.MOIDLimit then
		if math.random() < 0.2 then
			actor = RandomAHuman("Actors - Heavy" , faction);
		else
			actor = RandomAHuman("Actors - Light" , faction);
		end

		if actor then
			local mdl = self:GetModuleName(actor:GetModuleAndPresetName())
		
			-- probability to give a heavy weapon
			if math.random() < 0.35 then
				item = RandomHDFirearm("Weapons - Heavy" , mdl);
			else
				item = RandomHDFirearm("Weapons - Light" , mdl);
			end
			if item ~= nil then
				actor:AddInventoryItem(item);
			end

			if math.random() < 0.35 then
				item = RandomHDFirearm("Weapons - Secondary" , mdl);
				if item ~= nil then
					actor:AddInventoryItem(item);
				end
			end
			
			-- probability to give an explosive
			if math.random() < 0.15 then
				item = RandomTDExplosive("Bombs - Grenades" , mdl);
				if item ~= nil then
					actor:AddInventoryItem(item);
				end
			end

			if math.random() < 0.15 then
				item = RandomTDExplosive("Bombs - Grenades" , mdl);
				if item ~= nil then
					actor:AddInventoryItem(item);
				end
			end
			
			actor.AIMode = aimode;
			actor.Team = team;
			
			if pos ~= nil then
				actor.Pos = pos;
				if actor.Pos.X > SceneMan.Scene.Width / 2 then
					actor.HFlipped = true
				else
					actor.HFlipped = false
				end
				MovableMan:AddActor(actor);
				return actor;
			else
				return actor;
			end
		end
	end
	
	return nil;
end
-----------------------------------------------------------------------------------------
-- Brain selection and gameover conditions check
-----------------------------------------------------------------------------------------
function BlockWars:DoBrainSelection()
	if not (self.ActivityState == Activity.OVER) then
		-- Iterate through all human players
		for player = 0, self.PlayerCount - 1 do
			-- The current player's team
			local team = self:GetTeamOfPlayer(player);
	
			-- Make sure the game is not already ending
			-- Check if any player's brain is dead
			local brain = self:GetPlayerBrain(player);
			if not brain or not MovableMan:IsActor(brain) or not brain:HasObjectInGroup("Brains") then
				self:SetPlayerBrain(nil, player);
				-- Try to find a new unasigned brain this player can use instead, or if his old brain entered a craft
				local newBrain = MovableMan:GetUnassignedBrain(team);
				-- Found new brain actor, assign it and keep on truckin'
				if newBrain and self.braindead[player] == false then
					self:SetPlayerBrain(newBrain, player);
					self.AssignedBrains[player] = newBrain;
					self:SwitchToActor(newBrain, player, team);
				else
					FrameMan:SetScreenText("Commander died!", player, 333, -1, false);
					self.braindead[player] = true;
					
					-- Now see if all brains of self player's team are dead, and if so, end the game
					--if not MovableMan:GetClosestBrainActor(team) then
					self.WinnerTeam = self:OtherTeam(team);
					ActivityMan:EndActivity();
					--end
				end
			else
				-- Update the observation target to the brain, so that if/when it dies, the view flies to it in observation mode
				self:SetObservationTarget(brain.Pos, player)
			end -- 
		end --for
	end
end
-----------------------------------------------------------------------------------------
-- Spawn troops
-----------------------------------------------------------------------------------------
function BlockWars:ProcessJudges()
	if self.UseJudges then
		if self.JudgeTimer:IsPastSimMS(self.JudgeInterval) and not self.DreddAlive then
			if MovableMan:GetMOIDCount() < self.MOIDLimit then
				local craft = CreateACDropShip("Dropship MK1")
				if craft then
					craft.Team = -1;
					craft.AIMode = Actor.AIMODE_DELIVER;
					craft.Pos = Vector(self.JudgeLZ:GetCenterPoint().X, 0)
				
					for i = 1, 2 do
						local a = CreateAHuman("Street Judge")
						if a then
							a.AIMode = Actor.AIMODE_SENTRY
							a.Team = -1
							craft:AddInventoryItem(a)
						end
					end
				
					local a = CreateAHuman("Judge Dredd")
					if a then
						a.AIMode = Actor.AIMODE_SENTRY
						a.Team = -1
						craft:AddInventoryItem(a)
						self.NeedQuote = true;
					end
					
					MovableMan:AddActor(craft);
					self.ExpectedDropship = true
					self.JudgeTimer:Reset()
				end
			end
		end
		
		local toprotect = false;
		local protected = true
		
		if self.ExpectedDropship then
			for actor in MovableMan.Actors do
				if actor.PresetName == "Dropship MK1" then
					toprotect = true;
				end
			end
			
			if toprotect then
				for i = 1 , MovableMan:GetMOIDCount() - 1 do
					local mo = MovableMan:GetMOFromID(i);
					
					if mo.PresetName == "Dropship MK1" or mo.PresetName == "Drop Ship Engine A" or mo.PresetName == "Drop Ship Engine B" then
						mo.HitsMOs = false;
						mo.GetsHitByMOs = false;
						protected = true
					end
				end
				
				if protected then
					self.ExpectedDropship = false;
				end
			end
		end
		
		self.DreddAlive = false;
		
		for actor in MovableMan.Actors do
			if actor.Team == -1 then
				actor.Health = 100
				
				if actor.PresetName == "Judge Dredd" then
					self.DreddAlive = true;
					if self.NeedQuote then
						for i = 1, 3 do
							local em = CreateAEmitter("The Law")
							em.Pos = actor.Pos;
							MovableMan:AddParticle(em)
						end
						self.NeedQuote = false;
					end
				end
			end
		end
	end
end
-----------------------------------------------------------------------------------------
-- Spawn troops
-----------------------------------------------------------------------------------------
function BlockWars:Spawn(team , custompos)
	for i = 2, math.random(5) do
		if MovableMan:GetMOIDCount() < self.MOIDLimit then
			if math.random() < self:GetCrabToHumanSpawnRatio(PresetMan:GetModuleID(self.T[team].Faction)) then
				self:SpawnArmor(team , custompos , self.T[team].Faction , Actor.AIMODE_SENTRY)
			else
				local a = self:SpawnRandomInfantry(team , custompos , self.T[team].Faction , Actor.AIMODE_SENTRY)
				if a ~= nil then
					if math.random() < self.DiggerProbability then
						local d = CreateHDFirearm("Heavy Digger")
						if d ~= nil then
							a:AddInventoryItem(d)
						end
					end
				end
			end
		end
	end
end
-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
function BlockWars:UpdateActivity()
	-- Just check for intialization flags in update loop to avoid unnecessary function calls during all the mission
	if self.IsInitialized == nil then
		self.IsInitialized = false
	end
	
	if not self.IsInitialized then
		--Init mission if we're still not
		self:StartActivity()
	end	
	
   	-- Clear all objective markers, they get re-added each frame
   	self:ClearObjectivePoints();
   	
   	self:DoBrainSelection();
	self:UpdateBaseDefenceStats(); -- This MUST be called before any CPU decision making routines
	self:DispatchTroops(); -- This MUST be called before any CPU decision making routines

	for team = 0, 1 do
		if self.T[team].SpawnTimer:IsPastSimMS(self.T[team].SpawnInterval * self.T[team].SpawnCoeff) then
			self:Spawn(team , self.T[team].Spawn:GetCenterPoint());
			self.T[team].SpawnTimer:Reset();
		end
	end

	-- Periodically start rage to avoid cluttering
	if self.RageTimer:IsPastSimMS(self.RageInterval) then
		self.RageTimer:Reset();
		self.RageInterval = 30000 + (-10000 + math.random(20000));
		self:StartRage();
	end
	
	-- Add judges
	self:ProcessJudges()
	
	if not self.GenericTimer:IsPastSimMS(10000) then
		--Mark commander on start to warn player
		for player = 0, self.PlayerCount - 1 do
			if MovableMan:IsActor(self.AssignedBrains[player]) then
				if self.AssignedBrains[player]:HasObjectInGroup("Brains") then
					self:AddObjectivePoint("Protect commander!", self.AssignedBrains[player].AboveHUDPos, self:GetTeamOfPlayer(player), GameActivity.ARROWDOWN);
				end
			end -- if
		end -- for
	end
	
	self:PrintBaseDefenceStats();
	
	self:CheckWinningConditions();
	self:YSortObjectivePoints();--]]--
end
-----------------------------------------------------------------------------------------
-- Update base defence stats to make dispatch decisions later
-----------------------------------------------------------------------------------------
function BlockWars:UpdateBaseDefenceStats()
	self.UnitCount = 0;

	-- Reset stats
	for team = 0 , 1 do
		for defence = 1, 6 do
			self.T[team].DefEmpty[defence] = true;
		end
		self.T[team].ChamberEmpty = true;
		self.T[team].HallEmpty = true;
		self.T[team].Count = 0;
	end
	
	-- Update stats
	for actor in MovableMan.Actors do
		for team = 0 , 1 do
			if actor.PresetName ~= "Observer" then
				if actor.Team == team then
					for defence = 1, 6 do
						if self.T[team].Def[defence]:IsInside(actor.Pos) then
							self.T[team].DefEmpty[defence] = false;
						end
					end

					self.T[team].Count = self.T[team].Count + 1;
				else
					if self.T[team].Chamber:IsInside(actor.Pos) then
						self.T[team].ChamberEmpty = false;
					end

					if self.T[team].Hall:IsInside(actor.Pos) then
						self.T[team].HallEmpty = false;
					end
				end
			end
		end
	end
	
	for team = 0 , 1 do
		-- If enemies entered chamber send all for defence
		if self.T[team].ChamberEmpty == false then
			if self.T[team].ChamberEmptyLastTime == true then
				if self.Debug then
					print ("Chamber alert!!!");
				end
				for actor in MovableMan.Actors do
					if actor.Team == team and actor.PresetName ~= "Brain Case" then
						if actor.AIMODE ~= Actor.AIMODE_BRAINHUNT then
							self:AssignToBrainDefence(actor , team);
						end
					end
				end
			end
		else
			-- If enemies left chamber - revenge
			if self.T[team].ChamberEmptyLastTime == false then
				if self.Debug then
					print ("Revenge!!!");
				end
				for actor in MovableMan.Actors do
					if actor.Team == team and actor.PresetName ~= "Brain Case" then
						actor.AIMode = Actor.AIMODE_BRAINHUNT;
					end
				end
			end
		end

		-- Enforce spawn for team under siege
		if self.T[team].ChamberEmpty == false then
			self.T[team].SpawnCoeff = 0.5;
		else 
			self.T[team].SpawnCoeff = 1.0;
		end
		
		-- Reset chamber flags
		self.T[team].ChamberEmptyLastTime = self.T[team].ChamberEmpty;
	end

	-- Balance spawn depending on unit count
	if (self.T[self.PlayerTeam_1].Count > self.T[self.PlayerTeam_2].Count) then
		self.T[self.PlayerTeam_1].SpawnCoeff = self.T[self.PlayerTeam_1].SpawnCoeff + 0.25;
	else
		self.T[self.PlayerTeam_2].SpawnCoeff = self.T[self.PlayerTeam_2].SpawnCoeff + 0.25;
	end
end
---------------------------------------------------------------------
-- Print base defence stats to debug dispatch decisions
-----------------------------------------------------------------------------------------
function BlockWars:PrintBaseDefenceStats()
	if not self.Debug then 
		return;
	end

	local screenout = ""
	
	for team = 0 , 1 do
		local out;
		out = "";
		for defence = 1, 6 do
			if self.T[team].DefEmpty[defence] == false then
				out = out .. "X";
			else
				out = out .. "0";
			end
		end
	
		if self.T[team].ChamberEmpty == false then
			out = out .. "--X";
		else
			out = out .. "--0";
		end

		if self.T[team].HallEmpty == false then
			out = out .. "--X";
		else
			out = out .. "--0";
		end
		
		-- Print unit count and spawn rate coefficient`
		out = out.." ("..tostring(self.T[team].Count)..")  ("..tostring(self.T[team].SpawnCoeff)..")";
		screenout = screenout..out.."\n";
	end
	
	if self.Debug then
		FrameMan:SetScreenText(screenout, 0, 0, -1, false);
	end
end
---------------------------------------------------------------------
-- Update base defence stats to make dispatch decisions later
-----------------------------------------------------------------------------------------
function BlockWars:DispatchTroops()
	for actor in MovableMan.Actors do
		for team = 0 , 1 do
			if actor.Team == team then
				local assigned = false;
				-- Send troops to brain chamber
				-- Chamber uber alles
				if self.T[team].ChamberEmpty == false then
					if actor.AIMode ~= Actor.AIMODE_BRAINHUNT and actor.AIMode ~= Actor.AIMODE_GOTO and actor.PresetName ~= "Brain Case" then
						self:AssignToBrainDefence(actor , team);
						if self.T[team].Dispatch:IsInside(actor.Pos) then
							if self.Debug then
								actor:FlashWhite(500);
							end
						end
					end
				else
					-- Hall is important too
					if self.T[team].HallEmpty == false then
						if actor.AIMode ~= Actor.AIMODE_BRAINHUNT and actor.AIMode ~= Actor.AIMODE_GOTO and actor.PresetName ~= "Brain Case" then
							self:AssignToHallDefence(actor , team);
							if self.T[team].Dispatch:IsInside(actor.Pos) then
								actor:FlashWhite(500);
							end
						end
					else
						if actor.AIMode == Actor.AIMODE_SENTRY and self.T[team].Dispatch:IsInside(actor.Pos) then
							actor:FlashWhite(500);
							for defence = 6, 1, -1 do
								if self.T[team].DefEmpty[defence] == true then
									if self.GenericTimer.ElapsedSimTimeMS > self.T[team].LastDispatch[defence] + self.DispatchInterval then
										self:AssignToPoint(actor , team , defence);
										if self.Debug then
											actor:FlashWhite(500);
										end
										assigned = true;
										break;
									end -- if
								end -- if
							end -- for
							-- No empty points? Brainhunt!
							if assigned == false then
								if self.Debug then
									print ("ASSIGN: "..tostring(actor).." T:"..tostring(team).." Brainhunt");
								end
								actor:ClearAIWaypoints();
								actor.AIMode = Actor.AIMODE_BRAINHUNT;
								if self.Debug then
									actor:FlashWhite(500);
								end
							end --if
						end -- if
					end --if-else
				end --if
			end -- if
		end -- for
	end --for 
end -- func
-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function BlockWars:StartRage()
	print ("RAGE TIME!!!");
	for actor in MovableMan.Actors do
		if actor.PresetName ~= "Brain Case" and actor.Team ~= -1 then
			actor:ClearAIWaypoints();
			actor.AIMode = Actor.AIMODE_BRAINHUNT;
		end
	end
end
-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function BlockWars:AssignToHallDefence(act , team )
	if self.Debug then
		print ("ASSIGN:"..tostring(act).." T:"..tostring(team).." Hall");
	end
	act.AIMode = Actor.AIMODE_GOTO;
	act:ClearAIWaypoints();
	act:AddAISceneWaypoint(self.T[team].HallDef:GetCenterPoint());
	act:DrawWaypoints(self.DrawWaypoints);
end
-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function BlockWars:AssignToBrainDefence(act , team )
	if self.Debug then
		print ("ASSIGN:"..tostring(act).." T:"..tostring(team).." Defend");
	end
	act.AIMode = Actor.AIMODE_GOTO;
	act:ClearAIWaypoints();
	act:AddAISceneWaypoint(self.T[team].BrainRoom:GetCenterPoint());
	act:DrawWaypoints(self.DrawWaypoints);
	
	self:PrintBaseDefenceStats();
end
-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function BlockWars:AssignToPoint(act , team , pnt)
	if self.Debug then
		print ("ASSIGN: "..tostring(act).." T: "..tostring(team).." @ "..tostring(pnt));
	end
	act.AIMode = Actor.AIMODE_GOTO;
	act:ClearAIWaypoints();
	
	for p = 5 , pnt, -1 do
		act:AddAISceneWaypoint(self.T[team].Waypoint[p]:GetCenterPoint() + Vector(0 , 15));
	end
	act:AddAISceneWaypoint(self.T[team].Def[pnt]:GetCenterPoint() + Vector(0 , 15));	
	self.T[team].LastDispatch[pnt] = self.GenericTimer.ElapsedSimTimeMS;
	act:DrawWaypoints(self.DrawWaypoints);
	
	self:PrintBaseDefenceStats();
end
-----------------------------------------------------------------------------------------
-- Check if we are victorious
-----------------------------------------------------------------------------------------
function BlockWars:CheckWinningConditions()
	if self.CPUTeam > -1 and self.CPUBrainDead == false then
		if not MovableMan:IsActor(self.CPUBrain)then
			print ("Victory");
			-- Team 1 wins!
			self.CPUBrainDead = true;
			self.WinnerTeam = self:OtherTeam(self.CPUTeam);
			ActivityMan:EndActivity();
			-- Tell the script to stop running for brains as the game has ended.
			for player = 0, self.PlayerCount - 1 do
				self.braindead[player] = true;
			end
		else
			self:AddObjectivePoint("KILL", self.CPUBrain.AboveHUDPos, self.PlayerTeam_1, GameActivity.ARROWDOWN);
		end
	end
end
-----------------------------------------------------------------------------------------
-- That's all folks!
-----------------------------------------------------------------------------------------
